//Pressure sensor 7-bit I2C address: 0x60 (1100000)
//Distance sensor 7-bit I2C address: 0x29 (0101001)
//Motion sensor 7-bit I2C address:   0x75 (1110101)

#include <Wire.h>
#include "Keyboard.h"
#include "Mouse.h"
#include "Adafruit_VL6180X.h"
#include <Adafruit_MPL115A2.h>

//Global Variables
signed int deltaX=0, deltaY=0;
signed int deltaXB=0, deltaYB=0;
int tiltForceCompensation=0;
bool biteEnabled = false;
bool motion = false;

//Prototypes
void sensorInit();
void sensorGetPos();

Adafruit_VL6180X vl = Adafruit_VL6180X();
Adafruit_MPL115A2 mpl115a2;

void setup() 
{
  Wire.begin();
  Serial.begin(115200);
  // initialize mouse control:
  Mouse.begin();
  Keyboard.begin();
  mpl115a2.begin();
  sensorInit();

 //Measure startup force
  tiltForceCompensation = analogRead(0);     // read the force analog input pin
  tiltForceCompensation = (0-tiltForceCompensation);     // Zero offest for the sensor signal, arbitrary...

 
  // wait for serial port to open on native usb devices
  while (!Serial) {
    delay(1);
  }
  
  if (! vl.begin()) {
    Serial.println("Failed to find VL6180x sensor");
    while (1);
  }
}

void loop() 
{
  float lux;
  byte num=0;
  byte error;
  int range;
  int tiltForce;
  uint8_t status;
  float pressureKPA = 0, temperatureC = 0;  

  //hack to test pressure sensor - reset the i2c bus from scratch
  TWCR = 0; // reset TwoWire Control Register to default, inactive state
  Wire.begin();
  //delay(1);
  mpl115a2.begin();

  //Measure force and convert the result
  tiltForce = analogRead(0);     // read the force analog input pin
  tiltForce = tiltForce+tiltForceCompensation;     // Zero offest for the sensor signal, arbitrary...

  sensorGetPos();   //Loads motion delta into gloabal variables
  lux = vl.readLux(VL6180X_ALS_GAIN_5); //Read lux from TOF distance sensor
  range = vl.readRange();
  range = range - 18; //Zero offset for mouth use 
  if(range<0) {
    range = 0;   
  }

  status = vl.readRangeStatus();

  Serial.print("Enabled: ");
  if(tiltForce<=-100 || tiltForce >=100) //Light pressure front or back
  { 
    Serial.print("Yes\t");
    //Power up tracking if not already enabled
    if(biteEnabled == false)
    {
      biteEnabled = true;
      Wire.beginTransmission(117);
      Wire.write(0x06);  // configuration register
      Wire.write(0x17);  // POR value (not including PD_EnH bit)
      error = Wire.endTransmission();
      sensorInit(); //re-init the sensor (clear any old moves)
    }
    else //Already enabled, check 
    {
        Mouse.move(deltaX, deltaY);
    }
  }
  else //force not high or low enough to enable
  {
      Serial.print("\t");
  } 

  //Experimental scroll - more force -> faster scrolling
  if(tiltForce>=250) {
    Mouse.move(0,0,20);
  }
  else if(tiltForce<=-250) {
    Mouse.move(0,0,-20);
  }

  Serial.print("Click: ");
  if(tiltForce<=-320) //force sensor is pressed firmly front
  {
    Serial.print("Front\t");
    // perform mouse left click
    //Mouse.click(MOUSE_LEFT);
    //delay(10);     
  }
  else if(tiltForce>=630) //force sensor is pressed firmly back
  {
    Serial.print("Back\t");
    // perform mouse left click
    //Mouse.click(MOUSE_RIGHT);
    //delay(10);     
  }
  else {
    Serial.print("\t\t");
  }
  
  Serial.print("Motion: "); 
  if(motion) {
    Serial.print("Yes \t");
  }
  else {
    Serial.print(" \t"); 
  }

  if(motion) {
    Serial.print("X: "); Serial.print(deltaX); 
    Serial.print("\tY: "); Serial.print(deltaY);
  }
  else {
    Serial.print("X:     "); 
    Serial.print("\tY:    ");    
  }

  Serial.print("\tRange: ");
  if (status == VL6180X_ERROR_NONE) {
     Serial.print(range); Serial.print("\t");
  }
  else {
    Serial.print("\t\t");
  }

  Serial.print("Force: "); Serial.print(tiltForce);
  Serial.print("\tLux: "); Serial.print(lux);

  //Pressure and temperature sensor
  mpl115a2.getPT(&pressureKPA,&temperatureC);
  Serial.print("\tPressure(kPa): "); Serial.print(pressureKPA, 4);
  Serial.print(" \tTemp(*C): "); Serial.println(temperatureC, 1);
  
  //delay(5); 
}


//FUNCTIONS:

void sensorInit()
{
  byte num=0;
  byte error;
  
  // optical tracking sensor init 
  // check to see if the optical tracking sesnor is connected
  Wire.beginTransmission(117);
  Wire.write(0x00);  // address high byte
  error = Wire.endTransmission();
  
  // read 1 byte, from address 0
  Wire.requestFrom(117, 1);
  while(Wire.available())
  {
    num = Wire.read();
    if (num == 0x31)
    {
      //Serial.println("Sensor Found!");
      
      //Initial configuration sequence 
      // switch to register bank0
      Wire.beginTransmission(117);
      Wire.write(0x7F);  // configuration register address
      Wire.write(0x00);  // sensor reset value
      error = Wire.endTransmission(); 
      
      // reset the OTS
      Wire.beginTransmission(117);
      Wire.write(0x06);  // configuration register address
      Wire.write(0x97);  // sensor reset value
      error = Wire.endTransmission();
    
      // wait 1ms for sensor to reset
      delay(1);
      
      // re-init the i2c connection for a fresh start
      Wire.begin();
    
      // ensure the sensor has left the reset state.
      Wire.beginTransmission(117);
      Wire.write(0x06);  // configuration register address
      Wire.write(0x17);  // normal run mode value
      error = Wire.endTransmission();
    
      // disable write protect
      Wire.beginTransmission(117);
      Wire.write(0x09);  // address high byte
      Wire.write(0x5A);  // address low byte
      error = Wire.endTransmission();
    
      // set X-axis resolution
      Wire.beginTransmission(117);
      Wire.write(0x0D);  // address high byte
      //Wire.write(0x65);  // set X-axis resolution
      Wire.write(0x7A);  // set X-axis resolution
      error = Wire.endTransmission();
    
      // set Y-axis resolution
      Wire.beginTransmission(117);
      Wire.write(0x0E);  // address high byte
      //Wire.write(0xFF);  // set Y-axis resolution
      Wire.write(0x7A);  // set Y-axis resolution
      error = Wire.endTransmission();
      
      // set 12 bit output mode
      Wire.beginTransmission(117);
      Wire.write(0x19);  // address high byte
      Wire.write(0x04);  // set 8 bit output sensor mode
      error = Wire.endTransmission();
      
      // enable write protection
      Wire.beginTransmission(117);
      Wire.write(0x09);  // address high byte
      Wire.write(0x00);  // write protect value
      error = Wire.endTransmission();  
    }
    else
    {
      //Serial.println("Sensor not Found!");      
    }
  } 
}

void sensorGetPos()
{ 
  byte error;
  byte num=0;
  signed int deltaX_l=0, deltaY_l=0, deltaXY_h=0; 
  signed int deltaX_h=0, deltaY_h=0;

  //clear the global variables for position offset
  deltaX=0;
  deltaY=0;
  
  // Pass position information to host
  // read the motion register regardless of bite pressure
  Wire.beginTransmission(117);
  Wire.write(0x02);  // motion register
  error = Wire.endTransmission();
  // read 1 byte, from address 0
  Wire.requestFrom(117, 1);
  while(Wire.available())
  {
    num = Wire.read();  
    if(num & 0x80) //check motion bit in bit7
    {    
      motion = true;
      //Serial.println("motion detected!");
      //Read the motion registers
    
      // read the x data
      Wire.beginTransmission(117);
      Wire.write(0x03);  // Delta_X_Lo
      error = Wire.endTransmission();  
      // read 1 byte, from Delta_X_Lo
      Wire.requestFrom(117, 1);
      while(Wire.available())
      {
        deltaX_l = Wire.read();
      }
    
      // read the y data
      Wire.beginTransmission(117);
      Wire.write(0x04);  // Delta_Y_Lo
      error = Wire.endTransmission();   
      // read 1 byte, from Delta_Y_Lo
      Wire.requestFrom(117, 1);
      while(Wire.available())
      {
        deltaY_l = Wire.read();
      }

      // read the high nibble for X and Y motion (for 12 bit data)
      Wire.beginTransmission(117);
      Wire.write(0x12);  // Delta_XY_Hi
      error = Wire.endTransmission();  
      // read 1 byte, from Delta_XY_Hi
      Wire.requestFrom(117, 1);
      while(Wire.available())
      {
        deltaXY_h = Wire.read();
      }


      int shift = (sizeof(signed int) << 3) - 12 ;                                    

      deltaX_h = (deltaXY_h<<4) & 0xF00;
      deltaX_h = (deltaX_h << shift) >> shift ;
      //if(deltaX_h & 0x800)    deltaX_h |= 0xfffff000; // 12-bit data convert to 16-bit 
      
      deltaY_h = (deltaXY_h<<8) & 0xF00;
      deltaY_h = (deltaY_h << shift) >> shift ;
      //if(deltaY_h & 0x800)    deltaY_h |= 0xfffff000; // 12-bit data convert to 16-bit      
        
      deltaX = -(deltaX_h | deltaX_l);                 //inverse the data (depends on sensor's orientation and application)
      deltaY = -(deltaY_h | deltaY_l);                 //inverse the data (depends on sensor's orientation and application) 

    }
    else {
      motion = false; //motion bit is not set 
    }
  }
}

